/****************************************************************
 *                                                              *
 *  Copyright (C) 2003,2005 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#include "conversion.h"
#include "genxml.h"
#include "data.h"

CONVERSION::CONVERSION(CONVERSIONDATA* convdata)
{
	ConvData = convdata;
}

CONVERSION::~CONVERSION()
{
}

int CONVERSION::Run(HWND hOwner)
{
	HANDLE GedcomFileHandle,GenXMLFile;
	GEDCOMFILE* GedcomFile = NULL;
	GENXML* GenXML = NULL;
	GEDCOMDATA data;
	BOOL ConvOK;
	long iRepositories = 0;
	long iSources = 0;
	long iExcerpts = 0;
	long iEventtypes = 0;
	long iPersons = 0;
	long iAssertions = 0;
	long iTasks = 0;
	TCHAR* szMessage;
	DWORD BytesWritten;

	// Read the Gedcom file
	GedcomFileHandle = CreateFile(ConvData->szGedcomFile,GENERIC_READ,0,NULL,OPEN_EXISTING,0,NULL);
	if (GedcomFileHandle==INVALID_HANDLE_VALUE)
	{
		szMessage = ConvData->MsgProvider(ERROR_GEDCOMFILE);
		WriteFile(ConvData->hLogFile,szMessage,lstrlen(szMessage)*sizeof(TCHAR),&BytesWritten,NULL);
		return RETURN_GEDCOMFAILURE;
	}
	GedcomFile = new GEDCOMFILE(GedcomFileHandle,ConvData);
	ConvOK = data.Import(*GedcomFile,hOwner);
	delete GedcomFile;
	CloseHandle(GedcomFileHandle);
	if (!ConvOK)
		return RETURN_GEDCOMFAILURE;

	// Write the GenXML file
 	GenXMLFile = CreateFile(ConvData->szGenXMLFile,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,0,NULL);
	if (GenXMLFile==INVALID_HANDLE_VALUE)
		return RETURN_GENXMLFAILURE;
	GenXML = new GENXML(GenXMLFile,ConvData);
	GenXML->Header(data.Header,data.Submitters);
	iRepositories = GenXML->Repository(data.Repositories,data.Notes,data.NoteIDs);
	iSources = GenXML->Source(&data,ConvData->ObjectStatus);
	iExcerpts = GenXML->Excerpt(data.Sourcerefs,data.SourceIDs,data.Notes,data.NoteIDs);
	iEventtypes = GenXML->Eventtype(data.Eventtypes);
	iPersons = GenXML->Person(&data,ConvData->ObjectStatus);
	iAssertions = GenXML->Assertion(data.Persons,data.Families,data.PersonIDs,data.Notes,data.NoteIDs);
	iTasks = GenXML->Tasks(data.Todos,data.PersonIDs);
	//TODO
	GenXML->Total(iRepositories,iSources,iExcerpts,iEventtypes,iPersons,iAssertions,iTasks);
	delete GenXML;
	CloseHandle(GenXMLFile);
	return RETURN_OK;
}