/****************************************************************
 *                                                              *
 *  Copyright (C) 2003-2006 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#include "gedcom.h"
#include "data.h"
#include "ged2xml.h"

BOOL GEDCOMDATA::Import(GEDCOMFILE& file,HWND hOwner)
{
	GC_SUBMITTER* Submitter;
	GC_REPOSITORY* Repository;
	GC_SOURCE* Source;
	GC_PERSON* Person;
	GC_FAMILY* Family;
	GC_NOTEREC* Note;
	GC_OBJECTREC* Object;

	BOOL ConvOK = TRUE;
	if (!file.NextLine())
		return FALSE;
	if (!file.InterpretLine())
		return FALSE;
	// Import header
	if (file.GetTag()==TAG_HEAD)
		ConvOK = Header.Import(file);
	else
		file.PrintError(ERROR_HEADERMISSING,FALSE);
	// Detect character set
	if (Header.Char==TEXT("ANSI"))
		file.SetCharacterSet(CHARSET_ANSI);
	else if (Header.Char==TEXT("ANSEL"))
		file.SetCharacterSet(CHARSET_ANSEL);
	else if (Header.Char==TEXT("IBMPC"))
		file.SetCharacterSet(CHARSET_IBMPC);
	else if (Header.Char==TEXT("UTF-8") || Header.Char==TEXT("UTF8"))
		file.SetCharacterSet(CHARSET_UTF8);
	else if (Header.Char!=TEXT("UNICODE"))
	{
		file.PrintError(ERROR_UNKNOWNCHARSET,FALSE);
		file.SetCharacterSet(CHARSET_ANSEL);
	}
	// Import all data
	while (ConvOK && file.GetTag()!=TAG_TRLR)
	{
		if (hOwner && !SendMessage(hOwner,WM_CONVPROGRESS,0,file.GetDataRead()*100/file.GetGedcomFileSize()))
		{
			file.PrintError(ERROR_ABORTED,FALSE);
			ConvOK = FALSE;
			break;
		}
		if (!file.GetXRef()[0])
		{
			file.PrintError(ERROR_XREFMISSING,TRUE);
			ConvOK = FALSE;
		}
		else if (file.GetTag()==TAG_SUBM)
		{
			Submitter = new GC_SUBMITTER();
			ConvOK = Submitter->Import(file);
			Submitters.Add(Submitter);
		}
		else if (file.GetTag()==TAG_REPO)
		{
			Repository = new GC_REPOSITORY();
			RepositoryIDs.Add(file.GetXRef());
			ConvOK = Repository->Import(file);
			Repositories.Add(Repository);
		}
		else if (file.GetTag()==TAG_SOUR)
		{
			Source = new GC_SOURCE();
			SourceIDs.Add(file.GetXRef());
			ConvOK = Source->Import(file);
			Sources.Add(Source);
		}
		else if (file.GetTag()==TAG_INDI)
		{
			Person = new GC_PERSON();
			Person->SetID(PersonIDs.Add(file.GetXRef()));
			ConvOK = Person->Import(file,Eventtypes,Sources,Sourcerefs,Todos);
			Persons.Add(Person);
		}
		else if (file.GetTag()==TAG_FAM)
		{
			Family = new GC_FAMILY();
			ConvOK = Family->Import(file,Eventtypes,Sources,Sourcerefs);
			Families.Add(Family);
		}
		else if (file.GetTag()==TAG_NOTE)
		{
			Note = new GC_NOTEREC();
			ConvOK = Note->Import(file,&NoteIDs);
			Notes.Add(Note);
		}
		else if (file.GetTag()==TAG_OBJE)
		{
			Object = new GC_OBJECTREC();
			ConvOK = Object->Import(file,&ObjectIDs);//,&ObjectIDs);
			Objects.Add(Object);
		}
		else
		{
			file.PrintError(ERROR_ILLEGALTAG,TRUE);
			ConvOK = file.SkipStructure();
			if (!file.InterpretLine())
				ConvOK = FALSE;
		}
	}
	return ConvOK;
}

