/****************************************************************
 *                                                              *
 *  Copyright (C) 2003-2006 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#include <windows.h>
#include <Shlwapi.h>
#include "gc_date.h"
#include "ged2xml.h"
#include <strsafe.h>

// Only for CHAN
BOOL GC_SIMPLEDATE::Import(GEDCOMFILE& file)
{
	int level = file.GetLevel();
	if (!InterpretDate(file))
		return FALSE;
	if (!file.NextLine())
		return FALSE;
	if (!file.InterpretLine())
		return FALSE;
	// Check if next line is TIME on same or higher level
	if (file.GetTag()==TAG_TIME && file.GetLevel()>=level)
	{
		//if (file.ConvData->MessageLevel==MESSAGELEVEL_ALL)
		//	file.PrintError(ERROR_TIME,TRUE);
		TCHAR tmp[32];
		StringCbCopy(tmp,sizeof(tmp),file.GetText());
		TCHAR* ptr = wcstok(tmp,TEXT(":."));
		if (ptr)	// Find hours
		{
			hours = _wtoi(ptr);
			ptr = wcstok(NULL,TEXT(":."));
		}
		if (ptr)	// Find minutes
		{
			minutes = _wtoi(ptr);
			ptr = wcstok(NULL,TEXT(":."));
		}
		if (ptr)	// Find seconds
			seconds = _wtoi(ptr);
		if (!file.NextLine())
			return FALSE;
	}
	return TRUE;
}

// Interpret a standard gedcom date
BOOL GC_SIMPLEDATE::InterpretDate(GEDCOMFILE& file)
{
	static TCHAR szGregMonths[]=TEXT("JANFEBMARAPRMAYJUNJULAUGSEPOCTNOVDEC");
	static TCHAR szFrenchMonths[]=TEXT("VENDBRUMFRIMNIVOPLUVVENTGERMFLORPRAIMESSTHERFRUCCOMP");
	static TCHAR szHebrewMonths[]=TEXT("TSHCSHKSLTVTSHVADRADSNSNIYRSVNTMZAAVELL");
	TCHAR* szText = file.GetText();
	int x = 0;
	int tmp = 0;
	int i;
	day = 0;
	month = 0;
	year = 0;
	alternateyear = -1;
	while (szText[x]==' ')
		x++;
	// Check calendar
	if (!StrCmpN(szText,TEXT("@#DGREGORIAN@"),13))
	{
		x += 13;
		calendar = CALENDAR_GREGORIAN;
	}
	else if (!StrCmpN(szText,TEXT("@#DJULIAN@"),10))
	{
		x += 10;
		calendar = CALENDAR_JULIAN;
	}
	else if (!StrCmpN(szText,TEXT("@#DHEBREW@"),10))
	{
		x += 10;
		calendar = CALENDAR_HEBREW;
	}
	else if (!StrCmpN(szText,TEXT("@#DFRENCH R@"),12))
	{
		x += 12;
		calendar = CALENDAR_FRENCH;
	}
	else if (!StrCmpN(szText,TEXT("@#DUNKNOWN@"),11))
	{
		x += 11;
		calendar = CALENDAR_UNKNOWN;
	}
	else if (file.ConvData->AssumeGregorian)
		calendar = CALENDAR_GREGORIAN;
	else
		calendar = CALENDAR_UNKNOWN;
	while (szText[x]==' ')
		x++;
	// Look for day (or year)
	while (isdigit(szText[x]))
	{
		tmp *= 10;
		tmp += szText[x] - 0x30;
		x++;
	}
	// Check if read number is year
	if (szText[x]=='/')
	{
		year = tmp;
		goto altyear;
	}
	while (szText[x]==' ')
		x++;
	if (szText[x]=='B')
	{
		year = tmp;
		goto bc;
	}
	if (tmp>31 || !szText[x])
	{
		year = tmp;
		file.AddIndex(x);
		return TRUE;
	}
	day = tmp;
	// Look for month
	if (calendar==CALENDAR_FRENCH)
	{
		for (i=1;i<=13;i++)
		{
			if (!StrCmpNI(&szFrenchMonths[(i-1)*4],&szText[x],4))
				break;
		}
		x += 4;
		if (i>13)
		{
			year = tmp;
			file.AddIndex(x);
			return TRUE;
		}
	}
	else if (calendar==CALENDAR_HEBREW)
	{
		for (i=1;i<=13;i++)
		{
			if (!StrCmpNI(&szFrenchMonths[(i-1)*3],&szText[x],3))
				break;
		}
		x += 3;
		if (i>13)
		{
			year = tmp;
			file.AddIndex(x);
			return TRUE;
		}
	}
	else
	{
		for (i=1;i<=12;i++)
		{
			if (!StrCmpNI(&szGregMonths[(i-1)*3],&szText[x],3))
				break;
		}
		x += 3;
		if (i>12)
		{
			year = tmp;
			file.AddIndex(x);
			return TRUE;
		}
	}
	month = i;
	if (szText[x]==' ') x++;
	// Look for year
	while (isdigit(szText[x]))
	{
		year *= 10;
		year += szText[x] - 0x30;
		x++;
	}
altyear:
	// Look for alternate year
	if (szText[x]=='/')
	{
		x++;
		alternateyear = 0;
		while (isdigit(szText[x]))
		{
			alternateyear *= 10;
			alternateyear += szText[x] - 0x30;
			x++;
		}
		if (alternateyear>99)
			alternateyear %= 100;
	}
bc:
	while (szText[x]==' ')
		x++;
	file.AddIndex(x);
	// Is year before Christ?
	if (!StrCmpNI(file.GetText(),TEXT("BC"),2))
	{
		file.AddIndex(2);
		year = -year;
	}
	if (!StrCmpNI(file.GetText(),TEXT("B.C."),4))
	{
		file.AddIndex(4);
		year = -year;
	}
	return TRUE;
}

// Interpret a date on the form dd.mm.yyyy/aa
BOOL GC_SIMPLEDATE::InterpretEuropeanDate(GEDCOMFILE& file)
{
	TCHAR* szText = file.GetText();
	int x = 0;
	int tmp = 0;
	day = 0;
	month = 0;
	year = 0;
	alternateyear = -1;
	while (szText[x]==' ')
		x++;
	// Check calendar
	if (!StrCmpN(szText,TEXT("@#DGREGORIAN@"),13))
	{
		x += 13;
		calendar = CALENDAR_GREGORIAN;
	}
	else if (!StrCmpN(szText,TEXT("@#DJULIAN@"),10))
	{
		x += 10;
		calendar = CALENDAR_JULIAN;
	}
	else if (!StrCmpN(szText,TEXT("@#DHEBREW@"),10))
	{
		x += 10;
		calendar = CALENDAR_HEBREW;
	}
	else if (!StrCmpN(szText,TEXT("@#DFRENCH R@"),12))
	{
		x += 12;
		calendar = CALENDAR_FRENCH;
	}
	else if (!StrCmpN(szText,TEXT("@#DUNKNOWN@"),11))
	{
		x += 11;
		calendar = CALENDAR_UNKNOWN;
	}
	else if (file.ConvData->AssumeGregorian)
		calendar = CALENDAR_GREGORIAN;
	else
		calendar = CALENDAR_UNKNOWN;
	while (szText[x]==' ')
		x++;
	// Look for day
	while (isdigit(szText[x]))
	{
		tmp *= 10;
		tmp += szText[x] - 0x30;
		x++;
	}
	if (szText[x]!='.')
	{
		file.AddIndex(x);
		return FALSE;
	}
	x++;
	if (tmp>31)
	{
		file.AddIndex(x);
		return FALSE;
	}
	day = tmp;
	// Look for month
	tmp = 0;
	while (isdigit(szText[x]))
	{
		tmp *= 10;
		tmp += szText[x] - 0x30;
		x++;
	}
	if (szText[x]!='.')
	{
		file.AddIndex(x);
		return FALSE;
	}
	x++;
	if (tmp>13)
	{
		file.AddIndex(x);
		return FALSE;
	}
	month = tmp;
	// Look for year
	while (isdigit(szText[x]))
	{
		year *= 10;
		year += szText[x] - 0x30;
		x++;
	}
	// Look for alternate year
	if (szText[x]=='/')
	{
		x++;
		alternateyear = 0;
		while (isdigit(szText[x]))
		{
			alternateyear *= 10;
			alternateyear += szText[x] - 0x30;
			x++;
		}
		if (alternateyear>99)
			alternateyear %= 100;
	}
	while (szText[x]==' ')
		x++;
	file.AddIndex(x);
	// Is year before Christ?
	if (!StrCmpNI(file.GetText(),TEXT("BC"),2))
	{
		file.AddIndex(2);
		year = -year;
	}
	if (!StrCmpNI(file.GetText(),TEXT("B.C."),4))
	{
		file.AddIndex(4);
		year = -year;
	}
	return TRUE;
}

GC_DATE::GC_DATE()
{
	DateType = DATETYPE_NONE;
}

BOOL GC_DATE::Import(GEDCOMFILE& file)
{
	TCHAR* szText;
	TCHAR* szStart;
	TCHAR* ptr;
	int i,level;
	TCHAR tmp[32];

	level = file.GetLevel();
	// INT type date?
	szText = file.GetText();
	szStart = file.GetText();
	if (!StrCmpNI(szText,TEXT("INT"),3))
	{
		i = 0;
		while (szText[i] && szText[i]!='(')
			i++;
		if (!szText[i])
		{
			file.PrintError(ERROR_DATEFORMAT,TRUE);
			file.NextLine();
			return TRUE;
		}
		file.AddIndex(i);
	}
	// Text type date?
	szText = file.GetText();
	if (szText[0]=='(')
	{
		szText++;
		ptr = StrRChr(szText,NULL,')');
		if (ptr)
		{
			Text.Set(szText,ptr-szText);
			file.AddIndex(ptr-szText+2);
		}
		else
		{
			Text = szText;
			file.AddIndex(lstrlen(szText)+1);
		}
		DateType = DATETYPE_TEXT;
	}
	// Approximated type date?
	else if (!StrCmpNI(szText,TEXT("ABT "),4) || !StrCmpNI(szText,TEXT("CIR "),4) || !StrCmpNI(szText,TEXT("CA. "),4))
	{
		if (StrCmpNI(szText,TEXT("ABT "),4) && file.ConvData->MessageLevel==MESSAGELEVEL_ALL)
			file.PrintError(ERROR_CIRCA,TRUE);
		file.AddIndex(4);
		DateType = DATETYPE_ABT;
		Date1.InterpretDate(file);
	}
	else if (!StrCmpNI(szText,TEXT("CA "),3))
	{
		if (file.ConvData->MessageLevel==MESSAGELEVEL_ALL)
			file.PrintError(ERROR_CIRCA,TRUE);
		file.AddIndex(3);
		DateType = DATETYPE_ABT;
		Date1.InterpretDate(file);
	}
	else if (!StrCmpNI(szText,TEXT("CAL "),4) || !StrCmpNI(szText,TEXT("EST "),4))
	{
		file.AddIndex(4);
		DateType = DATETYPE_EST;
		Date1.InterpretDate(file);
	}
	// FROM or FROM ... TO type date?
	else if (!StrCmpNI(szText,TEXT("FROM "),5))
	{
		DateType = DATETYPE_FROM;
		file.AddIndex(5);
		Date1.InterpretDate(file);
		szText = file.GetText();
		if (!StrCmpNI(file.GetText(),TEXT("TO "),3))
		{
			DateType = DATETYPE_FROMTO;
			file.AddIndex(3);
			Date2.InterpretDate(file);
		}
	}
	// TO type date?
	else if (!StrCmpNI(szText,TEXT("TO "),3))
	{
		DateType = DATETYPE_TO;
		file.AddIndex(3);
		Date1.InterpretDate(file);
	}
	// BEF type date?
	else if (!StrCmpNI(szText,TEXT("BEF "),4))
	{
		DateType = DATETYPE_BEF;
		file.AddIndex(4);
		Date1.InterpretDate(file);
	}
	// AFT type date?
	else if (!StrCmpNI(szText,TEXT("AFT "),4))
	{
		DateType = DATETYPE_AFT;
		file.AddIndex(4);
		Date1.InterpretDate(file);
	}
	// BET type date?
	else if (!StrCmpNI(szText,TEXT("BET "),4))
	{
		DateType = DATETYPE_BET;
		file.AddIndex(4);
		Date1.InterpretDate(file);
		szText = file.GetText();
		if (StrCmpNI(szText,TEXT("AND "),4))
			file.PrintError(ERROR_DATEFORMAT,TRUE);
		else
		{
			file.AddIndex(4);
			Date2.InterpretDate(file);
		}
	}
	// BETWEEN = BET (typical TMG error)
	else if (!StrCmpNI(szText,TEXT("BETWEEN "),8))
	{
		if (file.ConvData->MessageLevel==MESSAGELEVEL_ALL)
			file.PrintError(ERROR_BETWEEN,TRUE);
		DateType = DATETYPE_BET;
		file.AddIndex(8);
		Date1.InterpretDate(file);
		szText = file.GetText();
		if (StrCmpNI(szText,TEXT("AND "),4))
			file.PrintError(ERROR_DATEFORMAT,TRUE);
		else
		{
			file.AddIndex(4);
			Date2.InterpretDate(file);
		}
	}
	// Exact type date?
	else
	{
		DateType = DATETYPE_EXACT;
		Date1.InterpretDate(file);
	}
	// Unknown characters?
	while (file.GetText()[0]==' ')
		file.AddIndex(1);
	if (file.GetText()[0] && file.ConvData->UserDefinedTags && file.GetSystem()==SYSTEM_BROTHERSKEEPERS)
	{
		file.InterpretLine();
		Date1.InterpretEuropeanDate(file);
	}
	if (file.GetText()[0])
	{
		if (file.ConvData->TextDates)
		{
			Text = szStart;
			DateType = DATETYPE_TEXT;
		}
		else
			file.PrintError(ERROR_DATEFORMAT,TRUE);
	}
	if (!file.NextLine())
		return FALSE;
	if (!file.InterpretLine())
		return FALSE;
	// Check if next line is TIME on same or higher level
	if (DateType==DATETYPE_EXACT && file.GetTag()==TAG_TIME && file.GetLevel()>=level)
	{
		if (file.ConvData->MessageLevel==MESSAGELEVEL_ALL)
			file.PrintError(ERROR_TIME,TRUE);
		StringCbCopy(tmp,sizeof(tmp),file.GetText());
		ptr = wcstok(tmp,TEXT(":."));
		if (ptr)	// Find hours
		{
			Date1.hours = _wtoi(ptr);
			ptr = wcstok(NULL,TEXT(":."));
		}
		if (ptr)	// Find minutes
		{
			Date1.minutes = _wtoi(ptr);
			ptr = wcstok(NULL,TEXT(":."));
		}
		if (ptr)	// Find seconds
			Date1.seconds = _wtoi(ptr);
		if (!file.NextLine())
			return FALSE;
	}
	return TRUE;
}

BOOL GC_CHANGE::Import(GEDCOMFILE& file)
{
	int TopLevel = file.GetLevel();

	if (!file.NextLine())
		return FALSE;
	if (!file.InterpretLine())
		return FALSE;
	while (file.GetLevel()>TopLevel)
	{
		switch(file.GetTag())
		{
		case TAG_DATE:
			if (!Date.Import(file))
				return FALSE;
			// Unknown characters?
			/*while (file.GetText()[0]==' ')
				file.AddIndex(1);
			if (file.GetText()[0])
				file.PrintError(ERROR_DATEFORMAT,TRUE);*/
			break;
		case TAG_NOTE:
			file.PrintError(ERROR_IGNORE,TRUE);
			if (!file.SkipStructure())
				return FALSE;
			break;
		default:
			if (!file.IllegalTag())
				return FALSE;
			break;
		}
		if (!file.InterpretLine())
			return (FALSE);
	}
	return (TRUE);
}