/****************************************************************
 *                                                              *
 *  Copyright (C) 2003-2009 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#include <windows.h>
#include <strsafe.h>
#include "gc_object.h"

BYTE CodeLookup[128] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,	// 0x00 - 0x0f
						0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,	// 0x10 - 0x1f
						0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,	// 0x20 - 0x2f
						0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x00,0x00,0x00,0x00,0x00,0x00,	// 0x30 - 0x3f
						0x00,0x0c,0x0d,0x0e,0x0f,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1a,	// 0x40 - 0x4f
						0x1b,0x1c,0x1d,0x1e,0x1f,0x20,0x21,0x22,0x23,0x24,0x25,0x00,0x00,0x00,0x00,0x00,	// 0x50 - 0x5f
						0x00,0x26,0x27,0x28,0x29,0x2a,0x2b,0x2c,0x2d,0x2e,0x2f,0x30,0x31,0x32,0x33,0x34,	// 0x60 - 0x6f
						0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,0x3c,0x3d,0x3e,0x3f,0x00,0x00,0x00,0x00,0x00};	// 0x70 - 0x7f
int cFiles = 0;

GC_OBJECTREC::~GC_OBJECTREC()
{
	//DeleteFile(szPath);
}

BOOL GC_OBJECTREC::Import(GEDCOMFILE& file,IDTABLE* ObjectIDs)
{
	BOOL result = TRUE;

	ObjectIDs->Add(file.GetXRef());
	file.NextLine();
	if (!file.InterpretLine())
		return (FALSE);
	while (file.GetLevel()>0)
	{
		switch(file.GetTag())
		{
		case TAG_FORM:
			StringCbCopy(szFormat,sizeof(szFormat),file.GetText());
			result = file.NextLine();
			break;
		case TAG_REFN:		// Ignore
		case TAG_RIN:
			file.NextLine();
			break;
		case TAG_NOTE:
			result = Note.ImportNote(file);
			break;
		case TAG_TITL:
			szTitle = file.GetText();
			result = file.NextLine();
			break;
		case TAG_BLOB:
			result = ImportCode(file);
			break;
		default:
			result = file.IllegalTag();
			break;
		}
		if (!result)
			return FALSE;
		if (!file.InterpretLine())
			return FALSE;
	}
	return (TRUE);
}

BOOL GC_OBJECTREC::ImportCode(GEDCOMFILE& file)
{
	STRING code;
	BOOL result = TRUE;

	int TopLevel = file.GetLevel();
	file.NextLine();
	if (!file.InterpretLine())
		return (FALSE);
	while (file.GetLevel()>TopLevel)
	{
		switch(file.GetTag())
		{
		case TAG_CONT:
			code.Add(file.GetText());
			result = file.NextLine();
			break;
		default:
			result = file.IllegalTag();
			break;
		}
		if (!result)
			return FALSE;
		if (!file.InterpretLine())
			return FALSE;
	}
	Decode(code.Get());
	return TRUE;
}

void GC_OBJECTREC::Decode(TCHAR* code)
{
	HANDLE hFile;
	TCHAR tmpPath[MAX_PATH];
	TCHAR tmp[6];
	int i;
	BYTE a,b,c,d;
	BYTE Decoded[3];
	DWORD BytesWritten;

	// Create file
	cFiles++;
	GetTempPath(sizeof(tmpPath),tmpPath);
	StringCbCat(tmpPath,sizeof(tmpPath),TEXT("OBJ"));
	StringCbPrintf(tmp,sizeof(tmp),TEXT("%05i"),cFiles);
	StringCbCat(tmpPath,sizeof(tmpPath),tmp);
	StringCbCat(tmpPath,sizeof(tmpPath),TEXT("."));
	StringCbCat(tmpPath,sizeof(tmpPath),szFormat);
	szPath = tmpPath;
	hFile = CreateFile(szPath.Get(),GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	// Decode
	i = 0;
	while (code[i] && code[i+1] && code[i+2] && code[i+3])
	{
		a = CodeLookup[code[i]];
		b = CodeLookup[code[i+1]];
		c = CodeLookup[code[i+2]];
		d = CodeLookup[code[i+3]];
		Decoded[0] = (a << 2) + (b >> 4);
		Decoded[1] = (b << 4) + (c >> 2);
		Decoded[2] = (c << 6) + d;
		WriteFile(hFile,Decoded,3,&BytesWritten,NULL);
		i += 4;
	}
	// Close file
	CloseHandle(hFile);
}

BOOL GC_OBJECT::Import(GEDCOMFILE& file)
{
	BOOL result = TRUE;

	int TopLevel = file.GetLevel();
	if (file.GetPointer()[0])
	{
		StringCbCopy(ObjectID,sizeof(ObjectID),file.GetPointer());
		file.NextLine();
		return TRUE;
	}
	file.NextLine();
	if (!file.InterpretLine())
		return (FALSE);
	while (file.GetLevel()>TopLevel)
	{
		switch(file.GetTag())
		{
		case TAG_FORM:
			StringCbCopy(szFormat,sizeof(szFormat),file.GetText());
			result = file.NextLine();
			break;
		case TAG_FILE:
			szPath = file.GetText();
			result = file.NextLine();
			break;
		case TAG_NOTE:
			result = Note.ImportNote(file);
			break;
		case TAG_TITL:
			szTitle = file.GetText();
			result = file.NextLine();
			break;
		default:
			result = file.IllegalTag();
			break;
		}
		if (!result)
			return FALSE;
		if (!file.InterpretLine())
			return FALSE;
	}
	return (TRUE);
}

