/****************************************************************
 *                                                              *
 *  Copyright (C) 2003,2005 Christoffer Owe                     *
 *  For conditions of distribution and use, see License.txt     *
 *                                                              *
 ****************************************************************/

#ifndef __GED2XML
#define __GED2XML

#include <windows.h>

// Return values
#define RETURN_OK				0
#define RETURN_GEDCOMFAILURE	1
#define RETURN_GENXMLFAILURE	2

// Error messages
#define ERROR_OK				0	// No error
#define ERROR_EOF				1	// End of file reached (without TRLR)
#define ERROR_USERDEFINED		2	// The line is ignored because of a user defined tag
#define ERROR_UNKNOWNTAG		3	// The tag is unknown (not standard)
#define ERROR_HEADERMISSING		4	// The header record is missing
#define ERROR_ILLEGALTAG		5	// Unexpected tag
#define ERROR_LINE				6	// The line does not conform to the Gedcom standard
#define ERROR_GEDCOMFILE		7	// The Gedcom file could not be opened
#define ERROR_XREFMISSING		8	// An XREF was expected on this line.
#define ERROR_XREFTOOLONG		9	// An XREF is limited to 20 characters according to the GEDCOM standard.
#define ERROR_DATEFORMAT		10	// Erroneous date
#define ERROR_ABORTED			11	// The conversion was aborted
#define ERROR_UNKNOWNCHARSET	12	// An unknown character set was detected, ANSEL is assumed
#define ERROR_BETWEEN			13	// Warning: BETWEEN detected in date, treated as BET
#define ERROR_CIRCA				14	// Warning: CA, CA. or CIR detected in date, treated as ABT
#define ERROR_TIME				15	// Warning: TIME detected after date, accepted but not standard GEDCOM
//#define ERROR_POINTERMISSING	16	// A pointer was expected after the tag
#define ERROR_IGNORE			100	// The line will be ignored.
// Default descriptions for event types
#define EVENTTYPE_BIRT		1000
#define EVENTTYPE_CHR		1001
#define EVENTTYPE_DEAT		1002
#define EVENTTYPE_BURI		1003
#define EVENTTYPE_CREM		1004
#define EVENTTYPE_ADOP		1005
#define EVENTTYPE_BAPM		1006
#define EVENTTYPE_BARM		1007
#define EVENTTYPE_BASM		1008
#define EVENTTYPE_BLES		1009
#define EVENTTYPE_CHRA		1010
#define EVENTTYPE_CONF		1011
#define EVENTTYPE_FCOM		1012
#define EVENTTYPE_ORDN		1013
#define EVENTTYPE_NATU		1014
#define EVENTTYPE_EMIG		1015
#define EVENTTYPE_IMMI		1016
#define EVENTTYPE_CENS		1017
#define EVENTTYPE_PROB		1018
#define EVENTTYPE_WILL		1019
#define EVENTTYPE_GRAD		1020
#define EVENTTYPE_RETI		1021
#define EVENTTYPE_RESI		1022
// Attributes
#define EVENTTYPE_CAST		1023
#define EVENTTYPE_DSCR		1024
#define EVENTTYPE_EDUC		1025
#define EVENTTYPE_IDNO		1026
#define EVENTTYPE_NATI		1027
#define EVENTTYPE_NCHI		1028
#define EVENTTYPE_NMR		1029
#define EVENTTYPE_OCCU		1030
#define EVENTTYPE_PROP		1031
#define EVENTTYPE_RELI		1032
#define EVENTTYPE_SSN		1033
#define EVENTTYPE_TITL		1034
// Family events
#define EVENTTYPE_ANUL		1035
#define EVENTTYPE_DIV		1036
#define EVENTTYPE_DIVF		1037
#define EVENTTYPE_ENGA		1038
#define EVENTTYPE_MARR		1039
#define EVENTTYPE_MARB		1040
#define EVENTTYPE_MARC		1041
#define EVENTTYPE_MARL		1042
#define EVENTTYPE_MARS		1043
// LDS tags
#define EVENTTYPE_BAPL		1044
#define EVENTTYPE_CONL		1045
#define EVENTTYPE_ENDL		1046
#define EVENTTYPE_SLGC		1047
#define EVENTTYPE_SLGS		1048
// User-defined tags
#define EVENTTYPE_EMAIL		1049
#define EVENTTYPE_UID		1050

#ifdef GED2XML_EXPORTS
#define GED2XML_API __declspec(dllexport)
#else
#define GED2XML_API __declspec(dllimport)
#endif

#if defined(__cplusplus)
extern "C"
{
#endif

#pragma pack(push,8)
typedef struct _CONVERSIONDATA
{
	int		StructSize;			// Size of this structure
	WCHAR*	szGedcomFile;		// Path to GEDCOM file.
	WCHAR*	szGenXMLFile;		// Path to new GenXML file. If the file exist, it will be overwritten.
	HANDLE	hLogFile;			// Handle to log file. May for example be standard out. But see LogFileIsConsole below.
	int		MessageLevel;		// See values below
	WCHAR*	(*MsgProvider)(int TextID);
								// Function that will provide texts for error messages and event types.
	BOOL	AssumeGivenName;	// Assume the name preciding the surname is a given name
	BOOL	UseStructuredNames;	// Use structured names instead of the value of the NAME tag
	BOOL	AssumeGregorian;	// Assume dates to be Gregorian when calendar is unspecified
	BOOL	TextDates;			// Treat erroneous dates as text
	BOOL	AliasNames;			// Read alias as NAME
	BOOL	UserDefinedTags;	// Try reading user-defined tags
	int		ObjectStatus;		// 0 - all objects internal, 1 - all objects external, 2 - objects internal or external as in the gedcom file
	int		EventInfoStatus;	// AGE,AGNC,CAUS: 0 - ignored and logged, 1 - ignored and not logged, 2 - stored as notes
	int		PersonInfoStatus;	// RESN,AFN,RIN an others:  0 - ignored and logged, 1 - ignored and not logged, 2 - stored as notes
	int		CharacterSet;		// 0 - ANSI, 1 - UTF-8, 2 - UNICODE
	BOOL	LogFileIsConsole;	// Must be set to TRUE is the log file is standard out to ensure correct handling of Unicode messages.
} CONVERSIONDATA;
#pragma pack(pop)

// Values for MessageLevel
#define MESSAGELEVEL_ALL		0	// Print all messages, including warnings
#define MESSAGELEVEL_SKIPIGNORE	1	// Don't print error messages for ignored lines
#define MESSAGELEVEL_SKIPERRORS	2	// Neither print error messages for known errors

#define WM_CONVPROGRESS		(WM_USER+7000)
#define WM_SAVEPROGRESS		(WM_USER+7001)

GED2XML_API int ConvertFile(CONVERSIONDATA*,HWND hOwner);
#define Convert(conv) ConvertFile(conv,NULL)

#if defined(__cplusplus)
}
#endif 

#endif
